#ifndef HARKTOOL4_UTILS_HARKIO2_HPP
#define HARKTOOL4_UTILS_HARKIO2_HPP

#include <string>
#include <initializer_list>
#include <boost/shared_ptr.hpp>
#include <boost/format.hpp>
#include <harkio2/harkio.hpp>

namespace harktool4
{

template <typename ExceptionType>
void Requires(
        boost::shared_ptr<harkio2::harkconfig::HarkConfigFile> hark_config_file,
        const std::string& id,
        std::initializer_list<harkio2::Version> versions)
{
    if (!hark_config_file)
    {
        throw ExceptionType("not HARK-config file");
    }
    else if (hark_config_file->GetId() != id)
    {
        throw ExceptionType((boost::format("requires %s, but given %s")
                    % id % hark_config_file->GetId()).str());
    }
    else
    {
        auto version = hark_config_file->GetVersion();
        for (auto it = versions.begin(); it != versions.end(); ++it)
        {
            if (version == *it)
            {
                return;
            }
        }
        throw ExceptionType((boost::format("unsupported %s version %s")
                    % id % version.ToString()).str());
    }
}

}

#endif // HARKTOOL4_UTILS_HARKIO2_HPP
