#include "ScatterPlot.h"





//------------------------------------------------//
//
//------------------------------------------------//
void ScatterPlot::setStyle(int plot_style)
{
    this->plot_style = plot_style;
}




//------------------------------------------------//
//
//------------------------------------------------//
void ScatterPlot::setTitle(std::string title)
{
    this->title = title;
}


//------------------------------------------------//
//
//------------------------------------------------//
void ScatterPlot::setLabels(std::string x, std::string y, std::string z)
{
    x_label = x;
    y_label = y;
    z_label = z;
}



//------------------------------------------------//
//
//------------------------------------------------//
std::string ScatterPlot::plot(std::string fullpath)
{

qDebug("ScatterPlot::plot start\n");

    std::string file = createFile(fullpath);

    if( file == "")
    {
        return NULL;
    }

    qint64 pid;

    int ret = QProcess::startDetached("gnuplot", QStringList() << "-persist" << QString(file.c_str()), QDir::currentPath(), &pid);

    if (!ret)
    {
        THROW_CRITICAL(QString("Command error: gnuplot --persist %1").arg(file.c_str()));
    }


    qDebug() << "PLOT file=" << file.c_str();

    return file;
}



//------------------------------------------------//
//
//------------------------------------------------//
std::string ScatterPlot::createFile(std::string fullpath)
{
    try
    {

        FILE *fp;
        std::ostringstream tmp_os;

        // テンポラリファイルオープン
        char filename[256];
        sprintf(filename, "%s/temp-XXXXXX", fullpath.c_str());

        int fd =  mkstemp(filename);
        fp = fdopen(fd, "w");

        if (!fp)
        {
            ScatterPlot::THROW_CRITICAL( QString("Can't open tempfile : %1").arg(filename));
            return NULL;
        }

        fp = fopen(filename, "w");

        // タイトル
        fprintf(fp, "set title '%s'\n", title.c_str());

        // 軸のタイトル
        fprintf(fp,"set xlabel '%s'\n", x_label.c_str());
        fprintf(fp,"set ylabel '%s'\n", y_label.c_str());
        fprintf(fp,"set zlabel '%s'\n", z_label.c_str());

        // XYZ軸
        fprintf(fp,"set xzeroaxis lt -1\n");
        fprintf(fp,"set yzeroaxis lt -1\n");
        fprintf(fp,"set zeroaxis lt -1\n");
        fprintf(fp,"set ticslevel 0\n"); // Z軸交点固定

        // プロットスタイル
        {
            std::string style_string;
            switch(plot_style) {
                case PlotStyle_UNDEFINE:
                    style_string = "";
                    break;
                case PlotStyle_DOTS:
                    style_string = "with dots";
                    break;
                case PlotStyle_POINTS:
                    style_string = "with points pt 7 ";
//                    style_string = "with points pt 5 ";
                    break;
                case PlotStyle_LINES:
                    style_string = "with lines";
                    break;
                case PlotStyle_LINESPOINTS:
                    style_string = "with linespoints";
                default:
                    break;
            }
            fprintf(fp, "splot '-' notitle %s\n", style_string.c_str());
        }

        // 内容の表示
        for (uint i = 0; i < lists->size(); i++)
        {
            double x = (*lists)[i][0];
            double y = (*lists)[i][1];
            double z = (*lists)[i][2];
            std::ostringstream tmp_os;
            tmp_os << x << " " << y << " " << z;
            fprintf(fp, "%s\n", tmp_os.str().c_str());
            tmp_os.str("");
        }
        fprintf(fp, "e\n");
        fprintf(fp, "pause -1\n");
        fclose(fp);

        return std::string(filename);

    }
    catch (const std::exception& e)
    {
         ScatterPlot::THROW_CRITICAL(e.what());
    }
    catch (...)
    {
        ScatterPlot::THROW_CRITICAL(("unexpected error"));
    }

}






//---------------------------------------------------//
//　エラーダイアログ
//---------------------------------------------------//
void ScatterPlot::THROW_CRITICAL(QString msg)
{
    QMessageBox  msgBox;
    msgBox.setIcon(QMessageBox::Critical);
    msgBox.setText(msg);
    msgBox.exec();
}

